@@ -9,11 +9,11 @@ from dvc_data.index import (
     DataIndexEntry,
     build,
     checkout,
-    collect,
     md5,
     read_db,
     read_json,
     save,
+    view,
     write_db,
     write_json,
 )
@@ -85,48 +85,14 @@ def test_fs(tmp_upath, odb, as_filesystem):
     ]
 
 
-def test_collect(tmp_upath, odb, as_filesystem):
-    (tmp_upath / "foo").write_bytes(b"foo\n")
-    (tmp_upath / "data").mkdir()
-    (tmp_upath / "data" / "bar").write_bytes(b"bar\n")
-    (tmp_upath / "data" / "baz").write_bytes(b"baz\n")
-
-    index = DataIndex(
-        {
-            ("foo",): DataIndexEntry(odb=odb, cache=odb),
-            ("data",): DataIndexEntry(odb=odb, cache=odb),
-        },
-    )
-    fs = as_filesystem(tmp_upath.fs)
-    collect(index, tmp_upath, fs)
-    assert index[("foo",)].meta == Meta(size=4)
-    assert index[("foo",)].fs == fs
-    assert index[("foo",)].path == str(tmp_upath / "foo")
-    assert index[("data",)].meta.isdir
-    assert index[("data",)].fs == fs
-    assert index[("data",)].path == str(tmp_upath / "data")
-    assert index[("data", "bar")].meta == Meta(size=4)
-    assert index[("data", "bar")].fs == fs
-    assert index[("data", "bar")].path == str(tmp_upath / "data" / "bar")
-    assert index[("data", "baz")].meta == Meta(size=4)
-    assert index[("data", "baz")].fs == fs
-    assert index[("data", "baz")].path == str(tmp_upath / "data" / "baz")
-
-
 def test_md5(tmp_upath, odb, as_filesystem):
     (tmp_upath / "foo").write_bytes(b"foo\n")
     (tmp_upath / "data").mkdir()
     (tmp_upath / "data" / "bar").write_bytes(b"bar\n")
     (tmp_upath / "data" / "baz").write_bytes(b"baz\n")
 
-    index = DataIndex(
-        {
-            ("foo",): DataIndexEntry(odb=odb, cache=odb),
-            ("data",): DataIndexEntry(odb=odb, cache=odb),
-        },
-    )
     fs = as_filesystem(tmp_upath.fs)
-    collect(index, tmp_upath, fs)
+    index = build(str(tmp_upath), fs)
     md5(index)
     assert index[("foo",)].hash_info == HashInfo(
         "md5",
@@ -166,22 +132,16 @@ def test_build(tmp_upath, as_filesystem):
 
     fs = as_filesystem(tmp_upath.fs)
     index = build(str(tmp_upath), fs)
-    assert index[("foo",)] == DataIndexEntry(
-        meta=Meta(size=4),
-        fs=fs,
-        path=str(tmp_upath / "foo"),
-    )
+    assert index[("foo",)].meta.size == 4
+    assert index[("foo",)].fs == fs
+    assert index[("foo",)].path == str(tmp_upath / "foo")
     assert index[("data",)].meta.isdir
-    assert index[("data", "bar")] == DataIndexEntry(
-        meta=Meta(size=4),
-        fs=fs,
-        path=str(tmp_upath / "data" / "bar"),
-    )
-    assert index[("data", "baz")] == DataIndexEntry(
-        meta=Meta(size=4),
-        fs=fs,
-        path=str(tmp_upath / "data" / "baz"),
-    )
+    assert index[("data", "bar")].meta.size == 4
+    assert index[("data", "bar")].fs == fs
+    assert index[("data", "bar")].path == str(tmp_upath / "data" / "bar")
+    assert index[("data", "baz")].meta.size == 4
+    assert index[("data", "baz")].fs == fs
+    assert index[("data", "baz")].path == str(tmp_upath / "data" / "baz")
 
 
 def test_checkout(tmp_upath, odb, as_filesystem):
@@ -242,3 +202,112 @@ def test_write_read(odb, tmp_path, write, read):
     for key, entry in new_index.iteritems():
         assert index[key].meta == entry.meta
         assert index[key].hash_info == entry.hash_info
+
+
+def test_view_keys(odb):
+    from dvc_data.index.view import _view_keys
+
+    index = DataIndex(
+        {
+            ("foo",): DataIndexEntry(
+                odb=odb,
+                hash_info=HashInfo(
+                    name="md5", value="d3b07384d113edec49eaa6238ad5ff00"
+                ),
+            ),
+            ("dir", "subdir", "bar"): DataIndexEntry(
+                odb=odb,
+                hash_info=HashInfo(
+                    name="md5",
+                    value="1f69c66028c35037e8bf67e5bc4ceb6a.dir",
+                ),
+            ),
+        }
+    )
+
+    prefixes, keys = _view_keys(index, lambda k: True)
+    assert prefixes == {("dir",), ("dir", "subdir")}
+    assert keys == {("foo",), ("dir", "subdir", "bar")}
+
+    prefixes, keys = _view_keys(index, lambda k: False)
+    assert prefixes == keys == set()
+
+    prefixes, keys = _view_keys(index, lambda k: "dir" in k)
+    assert prefixes == {("dir",), ("dir", "subdir")}
+    assert keys == {("dir", "subdir", "bar")}
+
+    def filter_fn(key):
+        # filter should not recurse into ("dir",) prefix
+        assert key != ("dir", "subdir")
+        return "foo" in key
+
+    prefixes, keys = _view_keys(index, filter_fn)
+    assert prefixes == set()
+    assert keys == {("foo",)}
+
+
+def test_view(odb):
+    expected_entry = (
+        DataIndexEntry(
+            odb=odb,
+            hash_info=HashInfo(
+                name="md5",
+                value="1f69c66028c35037e8bf67e5bc4ceb6a.dir",
+            ),
+        ),
+    )
+    expected_key = ("dir", "subdir", "bar")
+    index = DataIndex(
+        {
+            ("foo",): DataIndexEntry(
+                odb=odb,
+                hash_info=HashInfo(
+                    name="md5", value="d3b07384d113edec49eaa6238ad5ff00"
+                ),
+            ),
+            expected_key: expected_entry,
+        }
+    )
+    index_view = view(index, lambda k: "dir" in k)
+    assert {expected_key} == index_view.keys()
+    assert expected_key in index_view
+    assert ("foo",) not in index_view
+    assert len(index_view) == 1
+    assert [(expected_key, expected_entry)] == list(index_view.iteritems())
+    assert index_view[expected_key] == expected_entry
+    assert index_view[expected_key] is index[expected_key]
+
+
+def test_view_traverse(odb):
+    index = DataIndex(
+        {
+            ("foo",): DataIndexEntry(
+                odb=odb,
+                hash_info=HashInfo(
+                    name="md5", value="d3b07384d113edec49eaa6238ad5ff00"
+                ),
+            ),
+            ("dir", "subdir", "bar"): DataIndexEntry(
+                odb=odb,
+                hash_info=HashInfo(
+                    name="md5",
+                    value="1f69c66028c35037e8bf67e5bc4ceb6a.dir",
+                ),
+            ),
+        }
+    )
+    index_view = view(index, lambda k: "dir" in k)
+
+    keys = []
+
+    def node_factory(_, key, children, *args):
+        if key:
+            keys.append(key)
+        list(children)
+
+    index_view.traverse(node_factory)
+    assert keys == [
+        ("dir",),
+        ("dir", "subdir"),
+        ("dir", "subdir", "bar"),
+    ]
